@@ -1,18 +1,11 @@
-"""
-    weasyprint.tests.test_draw.test_column
-    --------------------------------------
-
-    Test how columns are drawn.
-
-"""
+"""Test how columns are drawn."""
 
 from ..testing_utils import assert_no_logs
-from . import assert_pixels
 
 
 @assert_no_logs
-def test_column_rule_1():
-    assert_pixels('solid', 5, 3, '''
+def test_column_rule_1(assert_pixels):
+    assert_pixels('''
         a_r_a
         a_r_a
         _____
@@ -22,7 +15,7 @@ def test_column_rule_1():
         div { columns: 2; column-rule-style: solid;
               column-rule-width: 1px; column-gap: 3px;
               column-rule-color: red }
-        body { margin: 0; font-size: 0; background: white}
+        body { margin: 0; font-size: 0 }
         @page { margin: 0; size: 5px 3px }
       </style>
       <div>
@@ -34,8 +27,8 @@ def test_column_rule_1():
 
 
 @assert_no_logs
-def test_column_rule_2():
-    assert_pixels('dotted', 5, 3, '''
+def test_column_rule_2(assert_pixels):
+    assert_pixels('''
         a_r_a
         a___a
         a_r_a
@@ -45,7 +38,7 @@ def test_column_rule_2():
         div { columns: 2; column-rule-style: dotted;
               column-rule-width: 1px; column-gap: 3px;
               column-rule-color: red }
-        body { margin: 0; font-size: 0; background: white}
+        body { margin: 0; font-size: 0 }
         @page { margin: 0; size: 5px 3px }
       </style>
       <div>
@@ -56,3 +49,34 @@ def test_column_rule_2():
         <img src=blue.jpg>
         <img src=blue.jpg>
       </div>''')
+
+
+@assert_no_logs
+def test_column_rule_span(assert_pixels):
+    assert_pixels('''
+        ___________
+        ___________
+        ___________
+        ___a_______
+        ___a_r_a___
+        ___a_r_a___
+        ___________
+        ___________
+        ___________
+    ''', '''
+      <style>
+        img { display: inline-block; width: 1px; height: 1px }
+        div { columns: 2; column-rule: 1px solid red; column-gap: 3px }
+        article { column-span: all }
+        body { margin: 0; font-size: 0 }
+        @page { margin: 3px; size: 11px 9px }
+      </style>
+      <div>
+        <article>
+          <img src=blue.jpg>
+        </article>
+        <img src=blue.jpg>
+        <img src=blue.jpg>
+        <img src=blue.jpg>
+        <img src=blue.jpg>
+      </div>''')
