@@ -520,9 +520,9 @@ def test_text_underline(assert_pixels):
     assert_pixels('''
         _____________
         _zzzzzzzzzzz_
-        _zRRRRRRRRRz_
-        _zRRRRRRRRRz_
-        _zBBBBBBBBBz_
+        _zsssssssssz_
+        _zsssssssssz_
+        _zuuuuuuuuuz_
         _zzzzzzzzzzz_
         _____________
     ''', '''
@@ -533,7 +533,7 @@ def test_text_underline(assert_pixels):
           margin: 2px;
         }
         body {
-          color: red;
+          color: rgba(255, 0, 0, 0.5);
           font-family: weasyprint;
           font-size: 3px;
           text-decoration: underline blue;
@@ -549,8 +549,8 @@ def test_text_overline(assert_pixels):
         _____________
         _zzzzzzzzzzz_
         _zzzzzzzzzzz_
-        _zRRRRRRRRRz_
-        _zRRRRRRRRRz_
+        _zsssssssssz_
+        _zsssssssssz_
         _zzzzzzzzzzz_
         _____________
     ''', '''
@@ -561,7 +561,7 @@ def test_text_overline(assert_pixels):
           margin: 2px;
         }
         body {
-          color: red;
+          color: rgba(255, 0, 0, 0.5);
           font-family: weasyprint;
           font-size: 3px;
           text-decoration: overline blue;
@@ -574,9 +574,9 @@ def test_text_line_through(assert_pixels):
     assert_pixels('''
         _____________
         _zzzzzzzzzzz_
-        _zRRRRRRRRRz_
         _zBBBBBBBBBz_
-        _zRRRRRRRRRz_
+        _zuuuuuuuuuz_
+        _zBBBBBBBBBz_
         _zzzzzzzzzzz_
         _____________
     ''', '''
@@ -587,10 +587,10 @@ def test_text_line_through(assert_pixels):
           margin: 2px;
         }
         body {
-          color: red;
+          color: blue;
           font-family: weasyprint;
           font-size: 3px;
-          text-decoration: line-through blue;
+          text-decoration: line-through rgba(255, 0, 0, 0.5);
         }
       </style>
       <div>abc</div>''')
@@ -601,9 +601,9 @@ def test_text_multiple_text_decoration(assert_pixels):
     assert_pixels('''
         _____________
         _zzzzzzzzzzz_
-        _zRRRRRRRRRz_
-        _zBBBBBBBBBz_
+        _zsssssssssz_
         _zBBBBBBBBBz_
+        _zuuuuuuuuuz_
         _zzzzzzzzzzz_
         _____________
     ''', '''
@@ -614,7 +614,7 @@ def test_text_multiple_text_decoration(assert_pixels):
           margin: 2px;
         }
         body {
-          color: red;
+          color: rgba(255, 0, 0, 0.5);
           font-family: weasyprint;
           font-size: 3px;
           text-decoration: underline line-through blue;
@@ -628,9 +628,9 @@ def test_text_nested_text_decoration(assert_pixels):
     assert_pixels('''
         _____________
         _zzzzzzzzzzz_
-        _zRRRRRRRRRz_
-        _zRRRBBBRRRz_
-        _zBBBBBBBBBz_
+        _zsssssssssz_
+        _zsssBBBsssz_
+        _zuuuuuuuuuz_
         _zzzzzzzzzzz_
         _____________
     ''', '''
@@ -641,7 +641,7 @@ def test_text_nested_text_decoration(assert_pixels):
           margin: 2px;
         }
         body {
-          color: red;
+          color: rgba(255, 0, 0, 0.5);
           font-family: weasyprint;
           font-size: 3px;
           text-decoration: underline blue;
