@@ -417,6 +417,38 @@ def test_column_span_9():
     assert column3.children[0].children[0].children[0].text == 'ghi'
 
 
+@assert_no_logs
+def test_column_span_balance():
+    page, = render_pages('''
+      <style>
+        @font-face { src: url(weasyprint.otf); font-family: weasyprint }
+        @page { margin: 0; size: 8px 5px }
+        body { font-family: weasyprint; font-size: 1px }
+        div { columns: 2; column-gap: 0; line-height: 1; column-fill: auto }
+        section { column-span: all }
+      </style>
+      <div>
+        abc def
+        <section>line1</section>
+        ghi jkl
+      </div>
+    ''')
+    html, = page.children
+    body, = html.children
+    div, = body.children
+    column1, column2, section, column3 = div.children
+    assert (column1.position_x, column1.position_y) == (0, 0)
+    assert (column2.position_x, column2.position_y) == (4, 0)
+    assert (section.position_x, section.position_y) == (0, 1)
+    assert (column3.position_x, column3.position_y) == (0, 2)
+
+    assert column1.children[0].children[0].children[0].text == 'abc'
+    assert column2.children[0].children[0].children[0].text == 'def'
+    assert section.children[0].children[0].text == 'line1'
+    assert column3.children[0].children[0].children[0].text == 'ghi'
+    assert column3.children[0].children[1].children[0].text == 'jkl'
+
+
 @assert_no_logs
 def test_columns_multipage():
     page1, page2 = render_pages('''
