@@ -7,7 +7,7 @@ from PIL import Image
 from weasyprint import HTML
 
 from .draw import assert_pixels_equal
-from .testing_utils import assert_no_logs, capture_logs, resource_filename
+from .testing_utils import assert_no_logs, capture_logs, resource_path
 
 
 @pytest.mark.xfail
@@ -15,7 +15,7 @@ from .testing_utils import assert_no_logs, capture_logs, resource_filename
 def test_acid2():
     # TODO: fails because of Ghostscript rendering
     def render(filename):
-        return HTML(resource_filename(filename)).render()
+        return HTML(resource_path(filename)).render()
 
     with capture_logs():
         # This is a copy of https://www.webstandards.org/files/acid2/test.html
