@@ -1,22 +1,17 @@
-"""
-    weasyprint.css.utils
-    --------------------
-
-    Utils for CSS properties.
-    See http://www.w3.org/TR/CSS21/propidx.html and various CSS3 modules.
-
-"""
+"""Utils for CSS properties."""
 
 import functools
 import math
+from abc import ABC, abstractmethod
 from urllib.parse import unquote, urljoin
 
 from tinycss2.color3 import parse_color
 
+from .. import LOGGER
 from ..urls import iri_to_uri, url_is_absolute
 from .properties import Dimension
 
-# http://dev.w3.org/csswg/css3-values/#angles
+# https://drafts.csswg.org/css-values-3/#angles
 # 1<unit> is this many radians.
 ANGLE_TO_RADIANS = {
     'rad': 1,
@@ -26,7 +21,7 @@ ANGLE_TO_RADIANS = {
 }
 
 # How many CSS pixels is one <unit>?
-# http://www.w3.org/TR/CSS21/syndata.html#length-units
+# https://www.w3.org/TR/CSS21/syndata.html#length-units
 LENGTHS_TO_PIXELS = {
     'px': 1,
     'pt': 1. / 0.75,
@@ -34,10 +29,10 @@ LENGTHS_TO_PIXELS = {
     'in': 96.,  # LENGTHS_TO_PIXELS['pt'] * 72
     'cm': 96. / 2.54,  # LENGTHS_TO_PIXELS['in'] / 2.54
     'mm': 96. / 25.4,  # LENGTHS_TO_PIXELS['in'] / 25.4
-    'q': 96. / 25.4 / 4.,  # LENGTHS_TO_PIXELS['mm'] / 4
+    'q': 96. / 25.4 / 4,  # LENGTHS_TO_PIXELS['mm'] / 4
 }
 
-# http://dev.w3.org/csswg/css-values/#resolution
+# https://drafts.csswg.org/css-values/#resolution
 RESOLUTION_TO_DPPX = {
     'dppx': 1,
     'dpi': 1 / LENGTHS_TO_PIXELS['in'],
@@ -102,6 +97,41 @@ class CenterKeywordFakeToken:
     unit = None
 
 
+class Pending(ABC):
+    """Abstract class representing property value with pending validation."""
+    # See https://drafts.csswg.org/css-variables-2/#variables-in-shorthands.
+    def __init__(self, tokens, name):
+        self.tokens = tokens
+        self.name = name
+        self._reported_error = False
+
+    @abstractmethod
+    def validate(self, tokens, wanted_key):
+        """Get validated value for wanted key."""
+        raise NotImplementedError
+
+    def solve(self, tokens, wanted_key):
+        """Get validated value or raise error."""
+        try:
+            if not tokens:
+                # Having no tokens is allowed by grammar but refused by all
+                # properties and expanders.
+                raise InvalidValues('no value')
+            return self.validate(tokens, wanted_key)
+        except InvalidValues as exc:
+            if self._reported_error:
+                raise exc
+            source_line = self.tokens[0].source_line
+            source_column = self.tokens[0].source_column
+            value = ' '.join(token.serialize() for token in tokens)
+            message = (exc.args and exc.args[0]) or 'invalid value'
+            LOGGER.warning(
+                'Ignored `%s: %s` at %d:%d, %s.',
+                self.name, value, source_line, source_column, message)
+            self._reported_error = True
+            raise exc
+
+
 def split_on_comma(tokens):
     """Split a list of tokens on commas, ie ``LiteralToken(',')``.
 
@@ -264,7 +294,7 @@ def parse_2d_position(tokens):
 def parse_position(tokens):
     """Parse background-position and object-position.
 
-    See http://dev.w3.org/csswg/css3-background/#the-background-position
+    See https://drafts.csswg.org/css-backgrounds-3/#the-background-position
     https://drafts.csswg.org/css-images-3/#propdef-object-position
 
     """
@@ -503,18 +533,16 @@ def check_string_or_element_function(string_or_element, token):
 
 
 def check_var_function(token):
-    function = parse_function(token)
-    if function is None:
-        return
-    name, args = function
-    if name == 'var' and args:
-        ident = args.pop(0)
-        if ident.type != 'ident' or not ident.value.startswith('--'):
-            return
-
-        # TODO: we should check authorized tokens
-        # https://drafts.csswg.org/css-syntax-3/#typedef-declaration-value
-        return ('var()', (ident.value.replace('-', '_'), args or None))
+    if function := parse_function(token):
+        name, args = function
+        if name == 'var' and args:
+            ident = args.pop(0)
+            # TODO: we should check authorized tokens
+            # https://drafts.csswg.org/css-syntax-3/#typedef-declaration-value
+            return ident.type == 'ident' and ident.value.startswith('--')
+        for arg in args:
+            if check_var_function(arg):
+                return True
 
 
 def get_string(token):
@@ -707,11 +735,6 @@ def get_content_list_token(token, base_url):
     if string is not None:
         return string
 
-    # <var>
-    var = check_var_function(token)
-    if var is not None:
-        return var
-
     # contents
     if get_keyword(token) == 'contents':
         return ('content()', 'text')
