@@ -10,7 +10,7 @@ are images with intrinsic dimensions.
 try:
     # Available in Python 3.9+
     from importlib.resources import files
-except ImportError:
+except ImportError:  # pragma: no cover
     # Deprecated in Python 3.11+
     from importlib.resources import read_text
 else:
@@ -19,7 +19,7 @@ else:
 
 import re
 
-from . import CSS, css
+from . import CSS, Attachment, css
 from .css import get_child_text
 from .css.counters import CounterStyle
 from .formatting_structure import boxes
@@ -29,9 +29,12 @@ from .urls import get_url_attribute
 
 HTML5_UA_COUNTER_STYLE = CounterStyle()
 HTML5_UA = read_text(css, 'html5_ua.css')
+HTML5_UA_FORM = read_text(css, 'html5_ua_form.css')
 HTML5_PH = read_text(css, 'html5_ph.css')
 HTML5_UA_STYLESHEET = CSS(
     string=HTML5_UA, counter_style=HTML5_UA_COUNTER_STYLE)
+HTML5_UA_FORM_STYLESHEET = CSS(
+    string=HTML5_UA_FORM, counter_style=HTML5_UA_COUNTER_STYLE)
 HTML5_PH_STYLESHEET = CSS(string=HTML5_PH)
 
 # https://html.spec.whatwg.org/multipage/#space-character
@@ -42,9 +45,6 @@ HTML_SPACE_SEPARATED_TOKENS_RE = re.compile(f'[^{HTML_WHITESPACE}]+')
 def ascii_lower(string):
     r"""Transform (only) ASCII letters to lower case: A-Z is mapped to a-z.
 
-    :param string: An Unicode string.
-    :returns: A new Unicode string.
-
     This is used for `ASCII case-insensitive
     <https://whatwg.org/C#ascii-case-insensitive>`_ matching.
 
@@ -63,15 +63,9 @@ def ascii_lower(string):
 
 
 def element_has_link_type(element, link_type):
-    """
-    Return whether the given element has a ``rel`` attribute with the
-    given link type.
-
-    :param link_type: Must be a lower-case string.
-
-    """
-    return any(ascii_lower(token) == link_type for token in
-               HTML_SPACE_SEPARATED_TOKENS_RE.findall(element.get('rel', '')))
+    """Return whether element has a ``rel`` attribute with given link type."""
+    tokens = HTML_SPACE_SEPARATED_TOKENS_RE.findall(element.get('rel', ''))
+    return any(ascii_lower(token) == link_type for token in tokens)
 
 
 # Maps HTML tag names to function taking an HTML element and returning a Box.
@@ -213,34 +207,6 @@ def handle_col(element, box, _get_image_from_uri, _base_url):
     return [box]
 
 
-@handler('th')
-@handler('td')
-def handle_td(element, box, _get_image_from_uri, _base_url):
-    """Handle the ``colspan``, ``rowspan`` attributes."""
-    if isinstance(box, boxes.TableCellBox):
-        # HTML 4.01 gives special meaning to colspan=0
-        # https://www.w3.org/TR/html401/struct/tables.html#adef-rowspan
-        # but HTML 5 removed it
-        # https://html.spec.whatwg.org/multipage/tables.html#attr-tdth-colspan
-        # rowspan=0 is still there though.
-        try:
-            box.colspan = max(int(element.get('colspan', '').strip()), 1)
-        except ValueError:
-            box.colspan = 1
-        try:
-            box.rowspan = max(int(element.get('rowspan', '').strip()), 0)
-        except ValueError:
-            box.rowspan = 1
-    return [box]
-
-
-@handler('a')
-def handle_a(element, box, _get_image_from_uri, base_url):
-    """Handle the ``rel`` attribute."""
-    box.is_attachment = element_has_link_type(element, 'attachment')
-    return [box]
-
-
 @handler('{http://www.w3.org/2000/svg}svg')
 def handle_svg(element, box, get_image_from_uri, base_url):
     """Handle ``<svg>`` elements.
@@ -315,11 +281,20 @@ def get_html_metadata(html):
             if url is None:
                 LOGGER.error('Missing href in <link rel="attachment">')
             else:
-                attachments.append((url, attachment_title))
-    return dict(title=title, description=description, generator=generator,
-                keywords=keywords, authors=authors,
-                created=created, modified=modified,
-                attachments=attachments, lang=lang, custom=custom)
+                attachment = Attachment(url=url, description=attachment_title)
+                attachments.append(attachment)
+    return {
+        'title': title,
+        'description': description,
+        'generator': generator,
+        'keywords': keywords,
+        'authors': authors,
+        'created': created,
+        'modified': modified,
+        'attachments': attachments,
+        'lang': lang,
+        'custom': custom,
+    }
 
 
 def strip_whitespace(string):
