@@ -6,7 +6,7 @@ box fragement.
 Boxes in the new tree have *used values* in their ``position_x``,
 ``position_y``, ``width`` and ``height`` attributes, amongst others.
 
-See http://www.w3.org/TR/CSS21/cascade.html#used-value
+See https://www.w3.org/TR/CSS21/cascade.html#used-value
 
 """
 
@@ -34,7 +34,7 @@ def initialize_page_maker(context, root_box):
     page_break = root_box.style['break_before']
 
     # TODO: take care of text direction and writing mode
-    # https://www.w3.org/TR/css3-page/#progression
+    # https://www.w3.org/TR/css-page-3/#progression
     if page_break == 'right':
         right_page = True
     elif page_break == 'left':
@@ -105,10 +105,12 @@ def layout_document(html, root_box, context, max_loops=8):
     This includes line breaks, page breaks, absolute size and position for all
     boxes. Page based counters might require multiple passes.
 
-    :param root_box: root of the box tree (formatting structure of the html)
-                     the pages' boxes are created from that tree, i.e. this
-                     structure is not lost during pagination
-    :returns: a list of laid out Page objects.
+    :param root_box:
+        Root of the box tree (formatting structure of the HTML). The page boxes
+        are created from that tree, this structure is not lost during
+        pagination.
+    :returns:
+        A list of laid out Page objects.
 
     """
     initialize_page_maker(context, root_box)
@@ -233,7 +235,7 @@ class LayoutContext:
         self.running_elements = defaultdict(lambda: defaultdict(lambda: []))
         self.current_page = None
         self.forced_break = False
-        self.broken_out_of_flow = []
+        self.broken_out_of_flow = {}
         self.in_column = False
 
         # Cache
@@ -252,7 +254,7 @@ class LayoutContext:
         self._excluded_shapes_lists.append(self.excluded_shapes)
 
     def finish_block_formatting_context(self, root_box):
-        # See http://www.w3.org/TR/CSS2/visudet.html#root-height
+        # See https://www.w3.org/TR/CSS2/visudet.html#root-height
         if root_box.style['height'] == 'auto' and self.excluded_shapes:
             box_bottom = root_box.content_box_y() + root_box.height
             max_shape_bottom = max([
@@ -285,15 +287,20 @@ class LayoutContext:
          4: ['Third Header', '3.5th Header']}
 
         Value depends on current page.
-        http://dev.w3.org/csswg/css-gcpm/#funcdef-string
+        https://drafts.csswg.org/css-gcpm/#funcdef-string
 
-        :param store: dictionary where the resolved value is stored.
-        :param page: current page.
-        :param name: name of the named string or running element.
-        :param keyword: indicates which value of the named string or running
-                        element to use. Default is the first assignment on the
-                        current page else the most recent assignment.
-        :returns: text for string set, box for running element
+        :param dict store:
+            Dictionary where the resolved value is stored.
+        :param page:
+            Current page.
+        :param str name:
+            Name of the named string or running element.
+        :param str keyword:
+            Indicates which value of the named string or running element to
+            use. Default is the first assignment on the current page else the
+            most recent assignment.
+        :returns:
+            Text for string set, box for running element.
 
         """
         if self.current_page in store[name]:
@@ -309,10 +316,9 @@ class LayoutContext:
                         for (string_name, _) in element.style['string_set']:
                             if string_name == name:
                                 return first_string
-                    if isinstance(element, boxes.ParentBox):
-                        if element.children:
-                            element = element.children[0]
-                            continue
+                    if element.children:
+                        element = element.children[0]
+                        continue
                     break
             elif keyword == 'last':
                 return last_string
@@ -327,13 +333,12 @@ class LayoutContext:
         """Add a footnote to the layout for this page."""
         self.footnotes.remove(footnote)
         self.current_page_footnotes.append(footnote)
-        self._update_footnote_area()
+        return self._update_footnote_area()
 
     def unlayout_footnote(self, footnote):
         """Remove a footnote from the layout and return it to the waitlist."""
-
-        # Handle unlayouting a footnote that hasn't been laid out yet (or has
-        # already been unlayout'd):
+        # TODO: Handle unlayouting a footnote that hasn't been laid out yet or
+        # has already been unlayouted
         if footnote not in self.footnotes:
             self.footnotes.append(footnote)
             if footnote in self.current_page_footnotes:
@@ -350,16 +355,24 @@ class LayoutContext:
 
     def _update_footnote_area(self):
         """Update the page bottom size and our footnote area height."""
-        if self.current_footnote_area.height != 'auto':
+        if self.current_footnote_area.height != 'auto' and not self.in_column:
             self.page_bottom += self.current_footnote_area.margin_height()
         self.current_footnote_area.children = self.current_page_footnotes
         if self.current_footnote_area.children:
             footnote_area = build.create_anonymous_boxes(
                 self.current_footnote_area.deepcopy())
-            footnote_area, _, _, _, _, _ = block_level_layout(
+            footnote_area = block_level_layout(
                 self, footnote_area, -inf, None,
-                self.current_footnote_area.page, True, [], [], [], False, None)
+                self.current_footnote_area.page)[0]
             self.current_footnote_area.height = footnote_area.height
-            self.page_bottom -= footnote_area.margin_height()
+            if not self.in_column:
+                self.page_bottom -= footnote_area.margin_height()
+            last_child = footnote_area.children[-1]
+            overflow = (
+                last_child.position_y + last_child.margin_height() >
+                footnote_area.position_y + footnote_area.margin_height() -
+                footnote_area.margin_bottom)
+            return overflow
         else:
             self.current_footnote_area.height = 0
+            return False
