@@ -8,7 +8,9 @@ from xml.etree import ElementTree
 from cssselect2 import ElementWrapper
 
 from ..urls import get_url_attribute
-from .bounding_box import bounding_box, is_valid_bounding_box
+from .bounding_box import (
+    EMPTY_BOUNDING_BOX, bounding_box, extend_bounding_box,
+    is_valid_bounding_box)
 from .css import parse_declarations, parse_stylesheets
 from .defs import (
     apply_filters, clip_path, draw_gradient_or_pattern, paint_mask, use)
@@ -17,7 +19,8 @@ from .path import path
 from .shapes import circle, ellipse, line, polygon, polyline, rect
 from .text import text
 from .utils import (
-    PointError, color, normalize, parse_url, preserve_ratio, size, transform)
+    PointError, alpha_value, color, normalize, parse_url, preserve_ratio, size,
+    transform)
 
 TAGS = {
     'a': text,
@@ -118,6 +121,16 @@ class Node:
         """Text after the XML node."""
         return self._etree_node.tail
 
+    @property
+    def display(self):
+        """Whether node should be displayed."""
+        return self.get('display') != 'none'
+
+    @property
+    def visible(self):
+        """Whether node is visible."""
+        return self.display and self.get('visibility') != 'hidden'
+
     def __iter__(self):
         """Yield node children, handling cascade."""
         for wrapper in self._wrapper:
@@ -178,10 +191,16 @@ class Node:
     def get_href(self, base_url):
         """Get the href attribute, with or without a namespace."""
         for attr_name in ('{http://www.w3.org/1999/xlink}href', 'href'):
-            url = get_url_attribute(self, attr_name, base_url)
+            url = get_url_attribute(
+                self, attr_name, base_url, allow_relative=True)
             if url:
                 return url
 
+    def del_href(self):
+        """Remove the href attributes, with or without a namespace."""
+        for attr_name in ('{http://www.w3.org/1999/xlink}href', 'href'):
+            self.attrib.pop(attr_name, None)
+
     @staticmethod
     def process_whitespace(string, preserve):
         """Replace newlines by spaces, and merge spaces if not preserved."""
@@ -222,7 +241,8 @@ class Node:
             self.pop_rotation(original_rotate, rotate)
         if self.text:
             trailing_space = self.text.endswith(' ')
-        for child_element in element.iter_children():
+        element_children = tuple(element.iter_children())
+        for child_element in element_children:
             child = child_element.etree_element
             if child.tag in ('{http://www.w3.org/2000/svg}tref', 'tref'):
                 child_node = Node(child_element, self._style)
@@ -243,13 +263,16 @@ class Node:
             if original_rotate and 'rotate' not in child_node:
                 child_node.pop_rotation(original_rotate, rotate)
             children.append(child_node)
-            if child.tail:
+            tail = self.process_whitespace(child.tail, preserve)
+            if text_root and child_element is element_children[-1]:
+                if not preserve:
+                    tail = tail.rstrip(' ')
+            if tail:
                 anonymous_etree = ElementTree.Element(
                     '{http://www.w3.org/2000/svg}tspan')
                 anonymous = Node(
                     ElementWrapper.from_xml_root(anonymous_etree), self._style)
-                anonymous._etree_node.text = self.process_whitespace(
-                    child.tail, preserve)
+                anonymous._etree_node.text = tail
                 if original_rotate:
                     anonymous.pop_rotation(original_rotate, rotate)
                 if trailing_space and not preserve:
@@ -278,6 +301,13 @@ class Node:
             str(rotate.pop(0) if rotate else original_rotate[-1])
             for i in range(len(self.text)))
 
+    def override_iter(self, iterator):
+        """Override node’s children iterator."""
+        # As special methods are bound to classes and not instances, we have to
+        # create and assign a new type.
+        self.__class__ = type(
+            'Node', (Node,), {'__iter__': lambda _: iterator})
+
 
 class SVG:
     """An SVG document."""
@@ -308,6 +338,7 @@ class SVG:
         self.text_path_width = 0
 
         self.parse_defs(self.tree)
+        self.inherit_defs()
 
     def get_intrinsic_size(self, font_size):
         """Get intrinsic size of the image."""
@@ -372,6 +403,8 @@ class SVG:
         # Update font size
         font_size = size(node.get('font-size', '1em'), font_size, font_size)
 
+        original_streams = []
+
         if fill_stroke:
             self.stream.push_state()
 
@@ -380,20 +413,18 @@ class SVG:
         if filter_:
             apply_filters(self, node, filter_, font_size)
 
-        # Create substream for opacity
-        opacity = float(node.get('opacity', 1))
-        if fill_stroke and 0 <= opacity < 1:
-            original_stream = self.stream
-            box = self.calculate_bounding_box(node, font_size)
-            if is_valid_bounding_box(box):
-                coords = (box[0], box[1], box[0] + box[2], box[1] + box[3])
-            else:
-                coords = (0, 0, self.inner_width, self.inner_height)
-            self.stream = self.stream.add_group(coords)
-
         # Apply transform attribute
         self.transform(node.get('transform'), font_size)
 
+        # Create substream for opacity
+        opacity = alpha_value(node.get('opacity', 1))
+        if fill_stroke and 0 <= opacity < 1:
+            original_streams.append(self.stream)
+            box = self.calculate_bounding_box(node, font_size)
+            if not is_valid_bounding_box(box):
+                box = (0, 0, self.inner_width, self.inner_height)
+            self.stream = self.stream.add_group(*box)
+
         # Clip
         clip_path = parse_url(node.get('clip-path')).fragment
         if clip_path and clip_path in self.paths:
@@ -404,8 +435,10 @@ class SVG:
                 width, height = self.point(
                     node.get('width'), node.get('height'), font_size)
                 self.stream.transform(a=width, d=height, e=x, f=y)
+            original_tag = clip_path._etree_node.tag
             clip_path._etree_node.tag = 'g'
             self.draw_node(clip_path, font_size, fill_stroke=False)
+            clip_path._etree_node.tag = original_tag
             # At least set the clipping area to an empty path, so that it’s
             # totally clipped when the clipping path is empty.
             self.stream.rectangle(0, 0, 0, 0)
@@ -415,19 +448,55 @@ class SVG:
             if new_ctm.determinant:
                 self.stream.transform(*(old_ctm @ new_ctm.invert).values)
 
-        # Manage display and visibility
-        display = node.get('display') != 'none'
-        visible = display and (node.get('visibility') != 'hidden')
+        # Handle text anchor
+        if node.tag == 'text':
+            text_anchor = node.get('text-anchor')
+            children = tuple(node)
+            if children and not node.text:
+                text_anchor = children[0].get('text-anchor')
+        if node.tag == 'text' and text_anchor in ('middle', 'end'):
+            group = self.stream.add_group(0, 0, 0, 0)  # BBox set after drawing
+            original_streams.append(self.stream)
+            self.stream = group
+
+        # Set text bounding box
+        if node.display and TAGS.get(node.tag) == text:
+            node.text_bounding_box = EMPTY_BOUNDING_BOX
 
         # Draw node
-        if visible and node.tag in TAGS:
+        if node.visible and node.tag in TAGS:
             with suppress(PointError):
                 TAGS[node.tag](self, node, font_size)
 
         # Draw node children
-        if display and node.tag not in DEF_TYPES:
+        if node.display and node.tag not in DEF_TYPES:
             for child in node:
                 self.draw_node(child, font_size, fill_stroke)
+                visible_text_child = (
+                    TAGS.get(node.tag) == text and
+                    TAGS.get(child.tag) == text and
+                    child.visible)
+                if visible_text_child:
+                    if not is_valid_bounding_box(child.text_bounding_box):
+                        continue
+                    x1, y1 = child.text_bounding_box[:2]
+                    x2 = x1 + child.text_bounding_box[2]
+                    y2 = y1 + child.text_bounding_box[3]
+                    node.text_bounding_box = extend_bounding_box(
+                        node.text_bounding_box, ((x1, y1), (x2, y2)))
+
+        # Handle text anchor
+        if node.tag == 'text' and text_anchor in ('middle', 'end'):
+            group_id = self.stream.id
+            self.stream = original_streams.pop()
+            self.stream.push_state()
+            if is_valid_bounding_box(node.text_bounding_box):
+                x, y, width, height = node.text_bounding_box
+                group.extra['BBox'][:] = (x, y, x + width, y + height)
+                x_align = width / 2 if text_anchor == 'middle' else width
+                self.stream.transform(e=-x_align)
+            self.stream.draw_x_object(group_id)
+            self.stream.pop_state()
 
         # Apply mask
         mask = self.masks.get(parse_url(node.get('mask')).fragment)
@@ -444,7 +513,7 @@ class SVG:
         # Apply opacity stream and restore original stream
         if fill_stroke and 0 <= opacity < 1:
             group_id = self.stream.id
-            self.stream = original_stream
+            self.stream = original_streams.pop()
             self.stream.set_alpha(opacity, stroke=True, fill=True)
             self.stream.draw_x_object(group_id)
 
@@ -497,12 +566,15 @@ class SVG:
             marker_node = self.markers.get(marker)
 
             # Calculate position, scale and clipping
+            translate_x, translate_y = self.point(
+                marker_node.get('refX'), marker_node.get('refY'),
+                font_size)
             if 'viewBox' in marker_node.attrib:
                 marker_width, marker_height = self.point(
                     marker_node.get('markerWidth', 3),
                     marker_node.get('markerHeight', 3),
                     font_size)
-                scale_x, scale_y, translate_x, translate_y = preserve_ratio(
+                scale_x, scale_y, _, _ = preserve_ratio(
                     self, marker_node, font_size, marker_width, marker_height)
 
                 clip_x, clip_y, viewbox_width, viewbox_height = (
@@ -539,11 +611,10 @@ class SVG:
                 if is_valid_bounding_box(box):
                     scale_x = scale_y = min(
                         marker_width / box[2], marker_height / box[3])
+                    translate_x /= scale_x
+                    translate_y /= scale_y
                 else:
                     scale_x = scale_y = 1
-                translate_x, translate_y = self.point(
-                    marker_node.get('refX'), marker_node.get('refY'),
-                    font_size)
                 clip_box = None
 
             # Scale
@@ -571,10 +642,9 @@ class SVG:
 
                 overflow = marker_node.get('overflow', 'hidden')
                 if clip_box and overflow in ('hidden', 'scroll'):
-                    self.stream.push_state()
                     self.stream.rectangle(*clip_box)
-                    self.stream.pop_state()
                     self.stream.clip()
+                    self.stream.end()
 
                 self.draw_node(child, font_size, fill_stroke)
                 self.stream.pop_state()
@@ -605,7 +675,7 @@ class SVG:
 
         # Get fill data
         fill_source, fill_color = self.get_paint(node.get('fill', 'black'))
-        fill_opacity = float(node.get('fill-opacity', 1))
+        fill_opacity = alpha_value(node.get('fill-opacity', 1))
         fill_drawn = draw_gradient_or_pattern(
             self, node, fill_source, font_size, fill_opacity, stroke=False)
         if fill_color and not fill_drawn:
@@ -616,7 +686,7 @@ class SVG:
 
         # Get stroke data
         stroke_source, stroke_color = self.get_paint(node.get('stroke'))
-        stroke_opacity = float(node.get('stroke-opacity', 1))
+        stroke_opacity = alpha_value(node.get('stroke-opacity', 1))
         stroke_drawn = draw_gradient_or_pattern(
             self, node, stroke_source, font_size, stroke_opacity, stroke=True)
         if stroke_color and not stroke_drawn:
@@ -710,6 +780,28 @@ class SVG:
         for child in node:
             self.parse_defs(child)
 
+    def inherit_defs(self):
+        """Handle inheritance of different defined elements lists."""
+        for defs in (self.gradients, self.patterns):
+            for element in defs.values():
+                self.inherit_element(element, defs)
+
+    def inherit_element(self, element, defs):
+        """Recursively handle inheritance of defined element."""
+        href = element.get_href(self.url)
+        if not href:
+            return
+        element.del_href()
+        parent = defs.get(parse_url(href).fragment)
+        if not parent:
+            return
+        self.inherit_element(parent, defs)
+        for key, value in parent.attrib.items():
+            if key not in element.attrib:
+                element.attrib[key] = value
+        if next(iter(element), None) is None:
+            element.override_iter(parent.__iter__())
+
     def calculate_bounding_box(self, node, font_size, stroke=True):
         """Calculate the bounding box of a node."""
         if stroke or node.bounding_box is None:
