@@ -1,10 +1,4 @@
-"""
-    weasyprint.svg.shapes
-    ---------------------
-
-    Draw simple shapes.
-
-"""
+"""Draw simple shapes."""
 
 from math import atan2, pi, sqrt
 
@@ -74,7 +68,7 @@ def rect(svg, node, font_size):
         ry = height / 2
 
     # Inspired by Cairo Cookbook
-    # http://cairographics.org/cookbook/roundedrectangles/
+    # https://cairographics.org/cookbook/roundedrectangles/
     ARC_TO_BEZIER = 4 * (2 ** .5 - 1) / 3
     c1, c2 = ARC_TO_BEZIER * rx, ARC_TO_BEZIER * ry
 
