@@ -19,6 +19,18 @@ PANGO_STRETCH = {
     'extra-expanded': pango.PANGO_STRETCH_EXTRA_EXPANDED,
     'ultra-expanded': pango.PANGO_STRETCH_ULTRA_EXPANDED,
 }
+# From https://drafts.csswg.org/css-fonts/#font-stretch-prop
+PANGO_STRETCH_PERCENT = {
+    50: pango.PANGO_STRETCH_ULTRA_CONDENSED,
+    62.5: pango.PANGO_STRETCH_EXTRA_CONDENSED,
+    75: pango.PANGO_STRETCH_CONDENSED,
+    87.5: pango.PANGO_STRETCH_SEMI_CONDENSED,
+    100: pango.PANGO_STRETCH_NORMAL,
+    112.5: pango.PANGO_STRETCH_SEMI_EXPANDED,
+    125: pango.PANGO_STRETCH_EXPANDED,
+    150: pango.PANGO_STRETCH_EXTRA_EXPANDED,
+    200: pango.PANGO_STRETCH_ULTRA_EXPANDED,
+}
 PANGO_WRAP_MODE = {
     'WRAP_WORD': pango.PANGO_WRAP_WORD,
     'WRAP_CHAR': pango.PANGO_WRAP_CHAR,
