@@ -5,11 +5,11 @@ from math import inf
 
 import pyphen
 
-from .constants import LST_TO_ISO, PANGO_STRETCH, PANGO_STYLE, PANGO_WRAP_MODE
+from .constants import LST_TO_ISO, PANGO_WRAP_MODE
 from .ffi import (
     ffi, gobject, pango, pangoft2, unicode_to_char_p, units_from_double,
     units_to_double)
-from .fonts import font_features
+from .fonts import font_features, get_font_description
 
 
 def line_size(line, style):
@@ -58,13 +58,13 @@ def first_line_metrics(first_line, text, layout, resume_at, space_collapse,
 
 class Layout:
     """Object holding PangoLayout-related cdata pointers."""
-    def __init__(self, context, font_size, style, justification_spacing=0,
+    def __init__(self, context, style, justification_spacing=0,
                  max_width=None):
         self.justification_spacing = justification_spacing
-        self.setup(context, font_size, style)
+        self.setup(context, style)
         self.max_width = max_width
 
-    def setup(self, context, font_size, style):
+    def setup(self, context, style):
         self.context = context
         self.style = style
         self.first_line_direction = 0
@@ -78,9 +78,6 @@ class Layout:
             pango.pango_font_map_create_context(font_map),
             gobject.g_object_unref)
         pango.pango_context_set_round_glyph_positions(pango_context, False)
-        self.layout = ffi.gc(
-            pango.pango_layout_new(pango_context),
-            gobject.g_object_unref)
 
         if style['font_language_override'] != 'normal':
             lang_p, lang = unicode_to_char_p(LST_TO_ISO.get(
@@ -97,31 +94,17 @@ class Layout:
 
         assert not isinstance(style['font_family'], str), (
             'font_family should be a list')
-        self.font = ffi.gc(
-            pango.pango_font_description_new(),
-            pango.pango_font_description_free)
-        family_p, family = unicode_to_char_p(','.join(style['font_family']))
-        pango.pango_font_description_set_family(self.font, family_p)
-        pango.pango_font_description_set_style(
-            self.font, PANGO_STYLE[style['font_style']])
-        pango.pango_font_description_set_stretch(
-            self.font, PANGO_STRETCH[style['font_stretch']])
-        pango.pango_font_description_set_weight(
-            self.font, style['font_weight'])
-        pango.pango_font_description_set_absolute_size(
-            self.font, units_from_double(font_size))
-        if style['font_variation_settings'] != 'normal':
-            string = ','.join(
-                f'{key}={value}' for key, value in
-                style['font_variation_settings']).encode()
-            pango.pango_font_description_set_variations(self.font, string)
-        pango.pango_layout_set_font_description(self.layout, self.font)
+        font_description = get_font_description(style)
+        self.layout = ffi.gc(
+            pango.pango_layout_new(pango_context),
+            gobject.g_object_unref)
+        pango.pango_layout_set_font_description(self.layout, font_description)
 
         text_decoration = style['text_decoration_line']
         if text_decoration != 'none':
             metrics = ffi.gc(
                 pango.pango_context_get_metrics(
-                    pango_context, self.font, self.language),
+                    pango_context, font_description, self.language),
                 pango.pango_font_metrics_unref)
             self.ascent = units_to_double(
                 pango.pango_font_metrics_get_ascent(metrics))
@@ -199,11 +182,20 @@ class Layout:
                 add_attr(0, len(bytestring), letter_spacing)
 
             if word_spacing:
+                if bytestring == b' ':
+                    # We need more than one space to set word spacing
+                    self.text = ' \u200b'  # Space + zero-width space
+                    text, bytestring = unicode_to_char_p(self.text)
+                    pango.pango_layout_set_text(self.layout, text, -1)
+
                 space_spacing = (
                     units_from_double(word_spacing) + letter_spacing)
                 position = bytestring.find(b' ')
+                # Pango gives only half of word-spacing on boundaries
+                boundary_positions = (0, len(bytestring) - 1)
                 while position != -1:
-                    add_attr(position, position + 1, space_spacing)
+                    factor = 1 + (position in boundary_positions)
+                    add_attr(position, position + 1, factor * space_spacing)
                     position = bytestring.find(b' ', position + 1)
 
             if word_breaking:
@@ -220,8 +212,7 @@ class Layout:
     def set_tabs(self):
         if isinstance(self.style['tab_size'], int):
             layout = Layout(
-                self.context, self.style['font_size'], self.style,
-                self.justification_spacing)
+                self.context, self.style, self.justification_spacing)
             layout.set_text(' ' * self.style['tab_size'])
             line, _ = layout.get_first_line()
             width, _ = line_size(line, self.style)
@@ -236,25 +227,16 @@ class Layout:
         pango.pango_layout_set_tabs(self.layout, array)
 
     def deactivate(self):
-        del self.layout, self.font, self.language, self.style
+        del self.layout, self.language, self.style
 
     def reactivate(self, style):
-        self.setup(self.context, style['font_size'], style)
+        self.setup(self.context, style)
         self.set_text(self.text, justify=True)
 
 
 def create_layout(text, style, context, max_width, justification_spacing):
-    """Return an opaque Pango layout with default Pango line-breaks.
-
-    :param text: Unicode
-    :param style: a style dict of computed values
-    :param max_width:
-        The maximum available width in the same unit as ``style['font_size']``,
-        or ``None`` for unlimited width.
-
-    """
-    layout = Layout(
-        context, style['font_size'], style, justification_spacing, max_width)
+    """Return an opaque Pango layout with default Pango line-breaks."""
+    layout = Layout(context, style, justification_spacing, max_width)
 
     # Make sure that max_width * Pango.SCALE == max_width * 1024 fits in a
     # signed integer. Treat bigger values same as None: unconstrained width.
@@ -420,8 +402,7 @@ def split_first_line(text, style, context, max_width, justification_spacing,
         soft_hyphen_indexes = [
             match.start() for match in re.finditer(soft_hyphen, next_word)]
         soft_hyphen_indexes.reverse()
-        dictionary_iterations = [
-            next_word[:i + 1] for i in soft_hyphen_indexes]
+        dictionary_iterations = [next_word[:i+1] for i in soft_hyphen_indexes]
         start_word = 0
     elif auto_hyphenation:
         dictionary_key = (lang, left, right, total)
@@ -448,19 +429,12 @@ def split_first_line(text, style, context, max_width, justification_spacing,
             new_first_line, index = new_layout.get_first_line()
             new_first_line_width, _ = line_size(new_first_line, style)
             new_space = max_width - new_first_line_width
-            hyphenated = (
-                index is None and (
-                    new_space >= 0 or
-                    first_word_part == dictionary_iterations[-1]))
+            hyphenated = index is None and (
+                new_space >= 0 or first_word_part == dictionary_iterations[-1])
             if hyphenated:
                 layout = new_layout
                 first_line = new_first_line
                 resume_index = len(new_first_line_text.encode())
-                if text[len(new_first_line_text)] == soft_hyphen:
-                    # Recreate the layout with no max_width to be sure that
-                    # we don't break before the soft hyphen
-                    pango.pango_layout_set_width(layout.layout, -1)
-                    resume_index += len(soft_hyphen.encode())
                 break
 
         if not hyphenated and not first_line_text:
